@@ -20,3 +20,13 @@ from i2.signatures import (
     call_forgivingly,
     call_somewhat_forgivingly,
 )
+
+from i2.multi_object import (
+    MultiObj,
+    MultiFunc,
+    Pipe,
+    FuncFanout,
+    FlexFuncFanout,
+    ParallelFuncs,
+    ContextFanout,
+)
