@@ -1,3 +1,4 @@
+"""Decorator tools"""
 import inspect
 from collections import defaultdict
 from collections.abc import Mapping
@@ -5,9 +6,9 @@ from contextlib import suppress
 from functools import wraps, partial, update_wrapper
 from inspect import Signature, signature, Parameter
 from itertools import chain
-from typing import Iterable
+from typing import Iterable, Callable
 
-from i2.signatures import ch_func_to_all_pk, Sig, tuple_the_args
+from i2.signatures import ch_func_to_all_pk, Sig, tuple_the_args, kind_forgiving_func
 
 # keep the imports below here because might be referenced (or take care of refs)
 from i2.signatures import ch_signature_to_all_pk, params_of, copy_func
@@ -53,11 +54,12 @@ def double_up_as_factory(decorator_func):
 
     def validate_decorator_func(decorator_func):
         first_param, *other_params = signature(decorator_func).parameters.values()
-        assert (
-            first_param.default is None
-        ), f'First argument of the decorator function needs to default to None. Was {first_param.default}'
+        assert first_param.default is None, (
+            f'First argument of the decorator function needs to default to None. '
+            f'Was {first_param.default}'
+        )
         assert all(
-            p.kind == p.KEYWORD_ONLY for p in other_params
+            p.kind in {p.KEYWORD_ONLY, p.VAR_KEYWORD} for p in other_params
         ), f'All arguments (besides the first) need to be keyword-only'
         return True
 
@@ -73,6 +75,71 @@ def double_up_as_factory(decorator_func):
     return _double_up_as_factory
 
 
+# TODO: Review, doc, and make public.
+#   Namely, it may be cleaner to have the `argname=argtrans, ...` interface
+#   and offer utils to make an argtrans be the combination of a condition and a trans!
+@double_up_as_factory
+def _conditional_arg_trans(func=None, **condition_and_trans_of_argname):
+    """
+
+    See also: ensure_iterable_args, for a more complex example of how to use it.
+    """
+
+    @wraps(func)
+    def _func(*args, **kwargs):
+        sig = Sig(func)
+        kwargs = sig.kwargs_from_args_and_kwargs(args, kwargs)
+        for argname, (condition, trans) in condition_and_trans_of_argname.items():
+            if argname in kwargs and condition(kwargs[argname]):
+                # Note: Using tuple instead of list because quicker & safer (non-mutable)
+                kwargs[argname] = trans(kwargs[argname])
+        args, kwargs = sig.args_and_kwargs_from_kwargs(kwargs)
+        return func(*args, **kwargs)
+
+    return _func
+
+
+def _tuplize(x):
+    return (x,)
+
+
+def _mk_type_condition(condition):
+    """If condition is a type, make an isinstance function to check for that type"""
+    curryable_isinstance = kind_forgiving_func(isinstance)
+    if isinstance(condition, type) or isinstance(condition, tuple):
+        type_condition = condition
+        condition = partial(curryable_isinstance, class_or_tuple=type_condition)
+    return condition
+
+
+def ensure_iterable_args(func=None, **condition_of_argname):
+    """Wrap a function so that specific arguments are assured to be iterable if
+    they meet specific conditions.
+
+    The condition, in the example below, is being a string.
+    Note that in general, the condition needs to be a boolean function.
+    The explicit form of our example would say `names=lambda x: isinstance(x, str)`,
+    but `ensure_iterable_args` allows the convenience of just specifying the type,
+    or a tuple of types, and the actually boolean function will be made for you.
+
+    >>> @ensure_iterable_args(names=str)
+    ... def greet_people(names, greeting='Hello'):
+    ...     for name in names:
+    ...         yield f"{greeting} {name}!"
+    >>> assert list(greet_people(['Alice', 'Bob'])) == ['Hello Alice!', 'Hello Bob!']
+    >>> assert list(greet_people('Alice')) == ['Hello Alice!']
+
+    Note that to decorate a function, you can also use the form:
+
+    >>> greet_people = ensure_iterable_args(greet_people, names=str)
+    """
+    condition_and_trans_of_argname = {
+        argname: (_mk_type_condition(condition), _tuplize)
+        for argname, condition in condition_of_argname.items()
+    }
+    return _conditional_arg_trans(func, **condition_and_trans_of_argname)
+
+
 def transparently_wrapped(func):
     @wraps(func)
     def transparently_wrapped_func(*args, **kwargs):
@@ -116,15 +183,6 @@ def mk_args_kwargs_merger(func):
     return merge_args_and_kwargs
 
 
-def ensure_iterable_of_callables(x):
-    if isinstance(x, Iterable):
-        all(callable(xx) for xx in x)
-        return x
-    else:
-        assert callable(x)
-        return (x,)
-
-
 def kwargs_for_func(*funcs, **kwargs):
     """
     :param funcs:
@@ -144,93 +202,6 @@ def kwargs_for_func(*funcs, **kwargs):
     return dict((func, Sig(func).source_kwargs(**kwargs)) for func in funcs)
 
 
-# TODO: Finish this!
-# TODO: Test the handling var positional and var keyword
-class MultiFunc:
-    """
-    Call multiple functions, using a pool of arguments that they will draw from.
-
-    >>> from i2.tests.objects_for_testing import formula1, sum_of_args, mult, add
-    >>> mf1 = MultiFunc(funcs=(formula1, mult, add))
-    >>> kwargs_for_func = mf1.kwargs_for_func(w=1, x=2, z=3, a=4, b=5)
-
-    What's this for? Well, the raison d'etre of `MultiFunc` is to be able to do this:
-
-    >>> assert add(a=4, b=5) == add(**kwargs_for_func[add])
-
-    This wouldn't work on all functions since some functions have position only arguments (e.g. ``formula1``).
-    Therefore ``MultiFunc`` holds a "normalized" form of the functions; namely one that handles such things as
-    postion only and varargs.
-
-    # TODO: Make this work!
-    #   Right now raises: TypeError: formula1() got some positional-only arguments passed as keyword arguments: 'w'
-    # >>> assert formula1(1, x=2, z=3) == mf1.normalized_funcs[formula1](**kwargs_for_func[formula1])
-
-    Note: In the following, it looks like ``MultiFunc`` instances return dicts whose keys are strings.
-    This is not the case.
-    The keys are functions: The same functions that were input.
-    The reason for not using functions is that when printed, they include their hash, which invalidates the doctests.
-
-    >>> def print_dict(d):  # just a util for this doctest
-    ...     from pprint import pprint
-    ...     pprint({k.__name__: d[k] for k in sorted(d, key=lambda x: x.__name__)})
-    >>> mf1 = MultiFunc(funcs=(formula1, mult, add))
-    >>> print_dict(mf1.kwargs_for_func(w=1, x=2, z=3, a=4, b=5)) # doctest: +NORMALIZE_WHITESPACE
-    {'add': {'a': 4, 'b': 5},
-     'formula1': {'w': 1, 'x': 2, 'z': 3},
-     'mult': {'x': 2}}
-
-    Oh, and you can actually see the signature of kwargs_for_func:
-
-    >>> from inspect import signature
-    >>> signature(mf1)
-    <Signature (w, x: float, a, y=1, z: int = 1, b: float = 0.0)>
-
-    >>> mf2 = MultiFunc(funcs=(formula1, mult, add, sum_of_args))
-    >>> print_dict(mf2.kwargs_for_func(w=1, x=2, z=3, a=4, b=5, args=(7,8), kwargs={'a': 42}, extra_stuff='ignore'))
-    {'add': {'a': 4, 'b': 5},
-     'formula1': {'w': 1, 'x': 2, 'z': 3},
-     'mult': {'x': 2},
-     'sum_of_args': {'kwargs': {'a': 4,
-                                'args': (7, 8),
-                                'b': 5,
-                                'extra_stuff': 'ignore',
-                                'kwargs': {'a': 42},
-                                'w': 1,
-                                'x': 2,
-                                'z': 3}}}
-
-    """
-
-    # FIXME: TODO: This does indeed change the signature, but not the functionality (position only still raise errors!)
-    def normalize_func(self, func):
-        return ch_func_to_all_pk(tuple_the_args(func))
-
-    def __init__(self, funcs=()):
-        self.funcs = ensure_iterable_of_callables(funcs)
-        self.sigs = {func: Sig(func) for func in self.funcs}
-        self.normalized_funcs = {func: self.normalize_func(func) for func in self.funcs}
-        multi_func_sig = Sig.from_objs(*self.normalized_funcs.values())
-        # TODO: Finish attempt to add **all_other_kwargs_ignored to the signature
-        # multi_func_sig = (Sig.from_objs(
-        #     *self.normalized_funcs.values(),
-        #     Parameter(name='all_other_kwargs_ignored', kind=Parameter.VAR_KEYWORD)))
-        multi_func_sig.wrap(self)
-        # multi_func_sig.wrap(self.kwargs_for_func)
-
-    def kwargs_for_func(self, *args, **kwargs):
-        return dict(
-            (func, self.sigs[func].source_kwargs(**kwargs)) for func in self.funcs
-        )
-
-    # TODO: Give it a signature (needs to be done in __init__)
-    # TODO: Validation of inputs
-    def __call__(self, *args, **kwargs):
-        return dict(
-            (func, self.sigs[func].source_kwargs(**kwargs)) for func in self.funcs
-        )
-
-
 def assert_attrs(attrs):
     """
     Asserts, at construction time, that the class contains a specific set of attributes
