@@ -1,3 +1,5 @@
+"""Analyzing what attributes of an input object a function actually uses"""
+
 import ast
 import inspect
 import dis
@@ -47,7 +49,7 @@ def _get_imports_from_ast_root(ast_root, recursive=False):
 
 
 def get_imports_from_obj(o, recursive=False):
-    """ Getting imports for an object (usually, module) """
+    """Getting imports for an object (usually, module)"""
     root = _get_ast_root_from(o)
     yield from _get_imports_from_ast_root(root, recursive)
 
@@ -81,6 +83,8 @@ def get_class_that_defined_method(method):
 
 
 def cls_and_method_name_of_method(method):
+    if isinstance(method, property):
+        return get_class_that_defined_method(method.fget), method.fget.__name__
     return get_class_that_defined_method(method), method.__name__
 
 
@@ -280,11 +284,16 @@ def attrs_used_by_method_computation(
     tracker.attrs_to_ignore = [
         func for func in dir(tracker) if callable(getattr(tracker, func))
     ]
+
     if hasattr(tracker, method_name):
-        candidate_method = getattr(tracker, method_name)
         # Now, we want to track attributes.
         with start_tracking(tracker):
-            candidate_method(**method_kwargs)
+            if isinstance(cls_method, property):
+                candidate_method = cls_method.fget
+                candidate_method(tracker)
+            else:
+                candidate_method = getattr(tracker, method_name)
+                candidate_method(**method_kwargs)
 
         return _process_duplicates(
             tracker.attr_used, remove_duplicates=remove_duplicates
